#include "i2c.h"

#define ClockSpeed 10000            //10kHz on SCL
#define I2C1_SLAVE_ADDRESS7 0x50    //0x50 Slave Address


int i2c_init(void) {

  GPIO_InitTypeDef GPIO_InitStructure;
  I2C_InitTypeDef  I2C_InitStructure;

  /* GPIOB Periph clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);

  /* I2C1 and I2C2 Periph clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);

  /* Configure I2C1 pins: SCL and SDA ----------------------------------------*/
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD; // Open Drain, I2C bus pulled high externally
  GPIO_Init(GPIOB, &GPIO_InitStructure);


  I2C_DeInit(I2C1);

   /* I2C1 configuration ------------------------------------------------------*/
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = ClockSpeed;

  I2C_Init(I2C1, &I2C_InitStructure);


 /* Enable I2C1 -------------------------------------------------------------*/
  I2C_Cmd(I2C1, ENABLE);


  return 0;
}


int i2c_write(char* buf, int buf_size, int offset) {


/* Send I2C1 START condition */
 I2C_GenerateSTART(I2C1, ENABLE);

 /* Test on I2C1 EV5 and clear it */
//   while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

 /* Send EEPROM slave Address for write */
 I2C_Send7bitAddress(I2C1, I2C1_SLAVE_ADDRESS7, I2C_Direction_Transmitter);

 /* Test on I2C1 EV6 and clear it */
//   while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));

 /* Send I2C1 EEPROM internal address */
 I2C_SendData(I2C1, offset); // 0x02 is config register

 /* Test on I2C1 EV8 and clear it */
//   while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

 /* Send I2C1 EEPROM data */
 I2C_SendData(I2C1, buf[offset]);//puts config register into continious measurement mode.

 /* Test on I2C1 EV8 and clear it */
//   while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

 /* Send I2C1 STOP Condition */
 I2C_GenerateSTOP(I2C1, ENABLE);

  return 0;
}

int i2c_read(char * buf, int buf_size, int offset) {

/* Send I2C1 START condition */
 I2C_GenerateSTART(I2C1, ENABLE);

 /* Test on I2C1 EV5 and clear it */
 while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));

 /* Send EEPROM slave Address for write */
 I2C_Send7bitAddress(I2C1, I2C1_SLAVE_ADDRESS7, I2C_Direction_Receiver);

 I2C_SendData(I2C1, offset); // 0x02 is config register

 buf[offset] = I2C_ReceiveData(I2C1);  //puts config register into continious measurement mode.


 /* Send I2C1 STOP Condition */
 I2C_GenerateSTOP(I2C1, ENABLE);

}


